package Sentinel::Sentinel;

use strict;
use vars qw( @ISA %AUTOFIELDS );

use Sentinel::Base;
@ISA = qw( Sentinel::Base );

use Sentinel::Host;
use Sentinel::Notify;

use Mail::Sender;

%AUTOFIELDS = (
	hosts => undef,
	notifies => undef,
	blocks => undef,
	slave_port => 9122,
	lib_host => '0.0.0.0',
	lib_port => 9123,
	sleep => 60
);

sub OBJ_NEW
{
	my $self = shift;
	$self -> hosts( [] );
	$self -> notifies( [] );
	$self -> blocks( {} ); # A hashref of arrayrefs of hashrefs?
	return( $self );
}

sub run_tests
{
	# Asks each host block to run the tests
	my $self = shift;

	for( @{ $self -> hosts() } )
	{
		$_ -> run_tests();
	}
}

sub add_host
{
	my $self = shift;
	my $addr = shift;
	my $name = shift;

	push( @{ $self -> hosts() }, Sentinel::Host -> new() );
	$self -> fatal( "Host has no address" ) unless $addr;
	$self -> last_host() -> addr( $addr );
	$self -> last_host() -> name( $name ) if $name;
	$self -> last_host() -> sentinel( $self );

	$self -> logmsg( "Host added: $addr" );
}

sub last_host
{
	return( shift() -> hosts() -> [ -1 ] );
}

sub add_notify
{
	my $self = shift;
	my $type = shift;
	push( @{ $self -> notifies() }, Sentinel::Notify -> new() );
	$self -> last_notify() -> type( $type );
	$self -> logmsg( "notify '$type'" );
}

sub last_notify
{
	return( shift() -> notifies() -> [ -1 ] );
}

sub send_notification
{
	my $self = shift;
	my $host = shift;
	my $mtype = shift;
	my $message = shift;

	my( $target, $type, $sender );

	for( @{ $self -> notifies() } )
	{
		# TODO: Handle net reference and doing multiple alert types
		$type = $_ -> type();
		$target = $_ -> target();
		$self -> logmsg( "Sending notification about $host to $target via $type" );

		if( $type eq "email" )
		{
			# TODO: Make this configurable
			$sender = new Mail::Sender {
				smtp => 'localhost',
				from => 'sentinel@localhost'
			};

			$sender -> MailMsg( {
				to => $target,
				subject => "$mtype: $host",
				msg => $message
			} );
		}elsif( $type eq "dialup" )
		{
			$self -> logmsg( "DIALUP NOT WORKING" );
		}else
		{
			$self -> logmsg( "Unknown type: $type" );
		}
	}
}

sub named_block
{
	return( shift() -> blocks() -> { shift() } );
}

sub add_named_block
{
	# Adds a test to a named block, if the block does not exist
	# it's automagically created
	my $self = shift;
	my $blocks = $self -> blocks();
	my $name = shift;
	my $tag = shift;
	my %attr = @_;

	$self -> logmsg( "Adding block tag for $name: $tag" );
	$blocks -> { $name } = [] unless defined( $blocks -> { $name } );
	push( @{ $blocks -> { $name } }, {
		tag => $tag,
		attr => \%attr } );
}

1;
